/*
 * Developed by Eugene A. Asahara, November-December 2007.
 * 
 * This application applies an algorithm to compress large many to many intermediate fact tables in Analysis Services cubes.
 * The algorithm is described in Erik Veermans blog: http://blogs.solidq.com/EN/Erik/Lists/Posts/Post.aspx?ID=1. 
 * It encapsulates a complex process and relatively complex T-SQL that wouldnt be of too much value to advanced SSAS folks 
 * applying this to a cube with just a couple of candidate many to many relationships. But perhaps there 
 * are many not-too-advanced SSAS folks out there tweaking cubes with a few or even a lot of such relationships.
 * 
/*============================================================================
  File:    Form1.cs

  Summary: User Interface for the Compression algorithm.

           Part of CompressManyToMany

  Date:    February 2008
------------------------------------------------------------------------------

  Copyright (C) Microsoft Corporation.  All rights reserved.

  This source code is intended only as a supplement to Microsoft
  Development Tools and/or on-line documentation.  See these other
  materials for detailed information regarding Microsoft code samples.

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Data.OleDb;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.AnalysisServices.AdomdClient;
using Microsoft.AnalysisServices;

namespace CompressManyToMany
{
    /// <summary>
    /// Should match the indexes of imageList1.
    /// </summary>
    internal enum ProgressIcons
    {
        None = -1,
        Error = 0,
        Warning = 1,
        Informational = 2,
        SQL=3,
        Done=4,
        Task=5
    }
    internal struct clsProgress
    {
        public clsProgress(string message, string details,ProgressIcons icon,string objectname)
        {
            Message = message;
            Details = details;
            TimeStamp = DateTime.Now;
            Icon = icon;
            ObjectName = objectname;
        }
        public string Message;
        public string Details;
        public DateTime TimeStamp;
        public ProgressIcons Icon;
        public string ObjectName;
    }
    
    public partial class Form1 : Form
    {
        private class HourGlass
        {
            public HourGlass()
            {
                Cursor.Current = Cursors.WaitCursor;
            }
            ~HourGlass()
            {
                Cursor.Current = Cursors.Default;
            }
        }

        private Server serv;
        private Cube selectedcube;

        private DataSourceView dsv
        {
            get
            {
                if (selectedcube != null)
                {
                    return selectedcube.DataSourceView;
                }
                return null;
            }
        }

        private const string DoubleClickToSeeText = "Double-click a line to see expanded text fields.";

        public Form1()
        {

            InitializeComponent();
            //
            toolTip1.SetToolTip(lstRelationshipWarnings, DoubleClickToSeeText);
            toolTip1.SetToolTip(lvProgress, DoubleClickToSeeText);
            toolTip1.SetToolTip(lstMaxItems, DoubleClickToSeeText);
            SetDoubleClickString(txtConnectionString);
            SetDoubleClickString(txtLinkToMG);
            SetDoubleClickString(txtDimLinkColumn);
            SetDoubleClickString(txtIM2IDLinkToMG);
            SetDoubleClickString(txtIM2IDLinkToDim);
            SetDoubleClickString(txtIM2DToMG);
            SetDoubleClickString(txtIMGDim);

            txtCommandTimeout.Text = clsM2M.CommandTimeout.ToString();
            //
            DisconnectSSASServer();     //Call to initialize the state of some controls.
        }
        private void SetDoubleClickString(Control control)
        {
            toolTip1.SetToolTip(control, "Double-click to see expanded text fields.");
            control.DoubleClick += new EventHandler(Display_textBoxText);
        }
        private void InitProgress()
        {
            lvProgress.Items.Clear();
        }
        private void cmdConnect_Click(object sender, EventArgs e)
        {
            Connect();
        }
        private void Connect()
        {
            HourGlass h = new HourGlass();
            if (serv != null) DisconnectSSASServer();
            try
            {
                AddProgress("Connecting to SSAS server.", string.Empty, txtServer.Text);
                serv = new Server();
                serv.Connect(txtServer.Text);
                AddProgress("Loading SSAS databases.", string.Empty, txtServer.Text);
                foreach (Database database in serv.Databases)
                {
                    cmbDatabase.Items.Add(database);
                }
                AddProgress(serv.Databases.Count.ToString()+" databases loaded.", string.Empty, txtServer.Text);
                cmbDatabase.Enabled = true;
                cmdConnect.Visible = false;
                cmdDisconnect.Visible = true;
            }
            catch (Exception ex)
            {
                AddProgress("Error connecting to SSAS server.", ex.Message, txtServer.Text,ProgressIcons.Error);
                MessageBox.Show("Error connecting to server.");
            }
        }
        private void Clear_cmbCubes()
        {
            cmbCube.Items.Clear();
            cmbCube.Text = string.Empty;
        }
        private void DisconnectSSASServer()
        {
            HourGlass h = new HourGlass();
            if (serv != null)
            {
                AddProgress("Disconnecting from SSAS server.", string.Empty, serv.Name);
                serv.Disconnect();
            }
            serv = null;
            //
            cmbDatabase.Enabled = false;
            cmbDatabase.Items.Clear();
            Clear_cmbCubes();
            cmbCube.Enabled = false;
            lvRelationships.Items.Clear();
            cmdAssessment.Enabled = false;
            cmdGenerateScripts.Enabled = false;
            cmdPopulate.Enabled = false;
            cmdConnect.Visible = true;
            cmdDisconnect.Visible = false;
        }

        private void cmbCube_Click(object sender, EventArgs e)
        {
        }
        private void DisplayCube()
        {
            HourGlass h = new HourGlass();
            AddProgress("Listing many to many relationships in the selected cube.", string.Empty, selectedcube.Name);
            lvRelationships.Items.Clear();
            grpDSV.Text = "Data Source View - " + dsv.Name;
            txtConnectionString.Text = dsv.DataSource.ConnectionString;
            clsM2M.ConnectionString = dsv.DataSource.ConnectionString;
            foreach (MeasureGroup mg in selectedcube.MeasureGroups)
            {
                foreach (MeasureGroupDimension mgd in mg.Dimensions)
                {
                    if (mgd.GetType()==typeof(ManyToManyMeasureGroupDimension))
                    {
                        ManyToManyMeasureGroupDimension m2m = (ManyToManyMeasureGroupDimension)mgd;
                        MeasureGroupDimension IntermediateDimension = GetIntermediateDimension(mg, mgd, m2m.MeasureGroup);
                        if (IntermediateDimension == null)
                        {
                            AddProgress("Cannot find an intermediate dimension. Skipping this relationship.", "A valid intermediate dimension has a Regular relationship to both the BaseMG and the Intermediate Fact Table.",string.Empty,ProgressIcons.Informational);
                        }
                        else
                        {
                            clsM2M cm2m = new clsM2M(mg, m2m, IntermediateDimension, dsv);
                            cm2m.Progress += new clsM2M.ProgressDelegate(ProgressFromM2M);
                            if (cm2m.InitClass())       //This is executed just after the instantiation of the class and the events have been set.
                            {
                                ListViewItem item1 = new ListViewItem(mg.Name);
                                if (IntermediateDimension == null)
                                {
                                    item1.SubItems.Add(string.Empty);
                                }
                                else
                                {
                                    item1.SubItems.Add(IntermediateDimension.CubeDimension.Name);
                                }
                                item1.SubItems.Add(m2m.MeasureGroup.Name);
                                item1.SubItems.Add(m2m.CubeDimension.Name);
                                item1.Tag = cm2m;
                                //The icon stating that this was already done takes precedence over
                                //the icon indicating warnings.
                                if (cm2m.ChangesApplied)
                                {
                                    //If this m2m was already done through this application, it will list, 
                                    //but we can't do anything to it.
                                    item1.ImageIndex = (int)ProgressIcons.Done;
                                }
                                else if (cm2m.Errors.Count > 0)
                                {
                                    item1.ImageIndex = (int)ProgressIcons.Warning;
                                }
                                lvRelationships.Items.Add(item1);
                            }
                            else
                            {
                                cm2m = null;
                            }
                        }
                    }
                }
            
            }
            if (lvRelationships.Items.Count == 0)
            {
               AddProgress("No many to many relationships were found in this cube.",string.Empty,selectedcube.Name,ProgressIcons.Warning);
            }
        }
        private void ProgressFromM2M(object sender, ProgressEventArgs e)
        {
            AddProgress(e.Progress.Message, e.Progress.Details, e.Progress.ObjectName, e.Progress.Icon);
        }
        /// <summary>
        /// The Intermediate dimension a dimension which the BaseMG and the IntermediateMG have in common and
        /// the relationships between this dimension and the BaseMG must be Regular and the relationship
        /// between the IntermediateMG and the dimension must be regular.
        /// This condition is temporary creating a limitation. Accomodating trios with other types
        /// of relationships will require more coding.
        /// </summary>
        /// <param name="IntermediateMG"></param>
        /// <param name="ManyDimension"></param>
        /// <param name="BaseMG"></param>
        /// <returns></returns>
        private MeasureGroupDimension GetIntermediateDimension(MeasureGroup BaseMG, MeasureGroupDimension ManyDimension, MeasureGroup IntermediateMG)
        {
            foreach (MeasureGroupDimension mgd in BaseMG.Dimensions)
            {
                if (mgd.CubeDimensionID != ManyDimension.CubeDimensionID && IntermediateMG.Dimensions.Contains(mgd.CubeDimensionID))
                {
                    //The attribute must have a Regular relationship to both the BaseMG and InterMG.
                    //However, a Degenerate relationship between BaseMG and Intermediate is OK as well.
                    if ((mgd.GetType() == typeof(RegularMeasureGroupDimension) || mgd.GetType() == typeof(DegenerateMeasureGroupDimension)) && (IntermediateMG.Dimensions[mgd.CubeDimensionID].GetType() == typeof(RegularMeasureGroupDimension)))
                        return mgd;
                }
            }
            return null;
        }
        private void cmbDatabase_SelectedIndexChanged(object sender, EventArgs e)
        {
            HourGlass h = new HourGlass();
            Clear_cmbCubes();
            if (cmbDatabase.SelectedItem != null)
            {
                Database database = (Database)cmbDatabase.SelectedItem;
                AddProgress("Loading cubes for the selected SSAS database.", string.Empty, database.Name);
                foreach (Cube cube in database.Cubes)
                {
                    cmbCube.Items.Add(cube);
                }
                AddProgress("Loaded "+database.Cubes.Count.ToString()+ " cubes for the selected SSAS database.", string.Empty, database.Name);
                cmbCube.Enabled = true;
                lvRelationships.Items.Clear();
                cmdGenerateScripts.Enabled = false;
                cmdPopulate.Enabled = false;
            }
        }

        private void cmbCube_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbCube.SelectedItem != null)
            {
                selectedcube = (Cube)cmbCube.SelectedItem;
                cmdGenerateScripts.Enabled = false;
                cmdPopulate.Enabled = false;
                DisplayCube();
            }

        }
        private void AddProgress(string message, string details,string objname)
        {
            AddProgress(message, details, objname, ProgressIcons.None);
        }
        private void AddProgress(string message, string details, string objname,ProgressIcons state)
        {
            ListViewItem li = new ListViewItem(message);
            li.SubItems.AddRange(new string[] { System.DateTime.Now.ToLongTimeString(), details, objname });
            li.Tag = new clsProgress(message, details,state,objname);
            if (state != ProgressIcons.None)
            {
                li.ImageIndex = (int)state;
            }
            lvProgress.Items.Add(li);
            //Scroll to the bottom of the list to ensure the user can see what's going on.
            lvProgress.EnsureVisible(lvProgress.Items.Count - 1);
            lvProgress.Refresh();
        }
        private void LoadRelationshipWarnings()
        {
            if (SelectedM2M == null) return;
            lstRelationshipWarnings.Items.Clear();
            foreach (clsProgress prog in SelectedM2M.Errors)
            {
                ListViewItem li = new ListViewItem(prog.Message);
                li.SubItems.AddRange(new string[] {prog.TimeStamp.ToLongTimeString(),prog.Details });
                li.Tag = prog;
                if (prog.Icon != ProgressIcons.None)
                {
                    li.ImageIndex = (int)prog.Icon;
                }
                lstRelationshipWarnings.Items.Add(li);
            }
        }
        private void lvRelationships_SelectedIndexChanged(object sender, EventArgs e)
        {
            HourGlass h = new HourGlass();
            DisplayAssessmentControls();
            if (SelectedM2M == null) return;    //Don't need a message to select an item here since this event will only be called if one is selected.
            txtLinkToMG.Text = clsM2M.FormatStringArray(SelectedM2M.LinkToBaseMG,", ");
            txtDimLinkColumn.Text = clsM2M.FormatStringArray(SelectedM2M.DimLinkColumn,", ");
            txtIM2IDLinkToMG.Text = clsM2M.FormatStringArray(SelectedM2M.IM2IDLinkToMG,", ");
            txtIM2IDLinkToDim.Text = clsM2M.FormatStringArray(SelectedM2M.IM2IDLinkToDim,", ");
            txtIM2DToMG.Text = SelectedM2M.IM2DToMG;
            txtIMGDim.Text = SelectedM2M.IMGDim;
            chkRemoveOldIntMG.Text = "Remove old Intermediate MG (" +SelectedM2M.OldIntDimName+ ")";
            LoadRelationshipWarnings();
            txtMatrixRealKeyFieldName.Text = SelectedM2M.MatrixRealKeyFieldName;
            txtMatrixIntKeyName.Text = SelectedM2M.MatrixIntKeyName;
            txtMatrixTableName.Text = SelectedM2M.MatrixTableName;
            SetButtons();
            cmdAssessment.Enabled = true;
        }
        private void cmbDatabase_Click(object sender, EventArgs e)
        {

        }

        private void Assessment()
        {
            if (SelectedM2M == null) return;    //Shouldn't be able to click this button unless something was chosen.
            HourGlass h = new HourGlass();
            SelectedM2M.Assessment();
            DisplayMessageIfWarnings();
            DisplayAssessmentControls();
        }
        private void DisplayMessageIfWarnings()
        {
            if (SelectedM2M == null) return;
            if (SelectedM2M.TaskCount > 0)
            {
                //Only reload warnings if new tasks,errors or warnings were encountered on the last operation.
                LoadRelationshipWarnings();
                tabControl2.SelectedTab = tbM2MErrors;
                MessageBox.Show("Be sure to check for warnings, errors, or tasks in the Progress List.", "Warnings During the Operation", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }
        private void DisplayAssessmentControls()
        {
            lstMaxItems.Items.Clear();
            cmdGenerateScripts.Enabled = false;
            cmdPopulate.Enabled = false;
            cmdAssessment.Enabled = false;
            if (SelectedM2M == null)
            {
                ClearAssessmentFields();
                return;
            }
            if (SelectedM2M.AssessmentDone)
            {
                txtRowsInBaseMG.Text = SelectedM2M.TableRows(Tables.BaseMeasureGroup).ToString();
                txtRowsIntMG.Text = SelectedM2M.TableRows(Tables.IntermediateMeasureGroup).ToString();
                txtRowsDim.Text = SelectedM2M.TableRows(Tables.ManyDimension).ToString();
                txtRowsIntDim.Text = SelectedM2M.TableRows(Tables.IntermediateDimension).ToString();
                //
                if (SelectedM2M.RankCounts != null)
                {
                    foreach (RankCount rc in SelectedM2M.RankCounts)
                    {
                        ListViewItem li = new ListViewItem(rc.Rank.ToString());
                        li.SubItems.Add(rc.Count.ToString());
                        li.SubItems.Add(rc.Description(SelectedM2M.TableName(Tables.BaseMeasureGroup),SelectedM2M.TableName(Tables.ManyDimension)));
                        lstMaxItems.Items.Add(li);
                    }
                }
                if (lstMaxItems.Items.Count == 0)
                {
                    AddProgress("No items are found.", string.Empty,SelectedM2M.ToString());
                }
                txtMaxLenManyKey.Text = SelectedM2M.MaxManyKeyLength.ToString();
                txtMatrixKeyParts.Text = SelectedM2M.MatrixKeyParts.ToString();
                txtMaxAcceptableOnManySide.Text = SelectedM2M.MaxItemsAcceptableOnManySide.ToString();
                if (SelectedM2M.RowsInNewIntermediateMG == clsM2M.UnsetValue)
                {
                    //The values were not assessed for various reasons. See clsM2M.Assessment() for code
                    //that exits the method. Reasons include:
                    //
                    //  1.  The number of items on the "many" side is excessive. For example, there could be
                    //      a large number of search words for each web visit. That would result in a large
                    //      intermediate measure group that would make this compression method worthless.
                    //  2.  There are no rows in the current intermediate measure group. This doesn't mean
                    //      the compression method is worthless. It just means we wouldn't be able to
                    //      calculate the compression ratio to assess the value of this method.
                    txtNewRowCount.Text = "Unset";
                    txtCompressionRatio.Text = "Unset";
                }
                else
                {
                    txtNewRowCount.Text = SelectedM2M.RowsInNewIntermediateMG.ToString();
                    txtCompressionRatio.Text = SelectedM2M.CompressionRatio.ToString();
                }
                txtMatrixHashKeyLength.Text = SelectedM2M.MatrixHashKeyLength.ToString();
                
            }
            else
            {
                ClearAssessmentFields();
            }
            SetButtons();
        }
        private void SetButtons()
        {
            if (SelectedM2M == null) return;
            cmdAssessment.Enabled = SelectedM2M.AssessmentOKtoRun;
            cmdPopulate.Enabled = SelectedM2M.PopulateMatrixOKtoRun(false,chkGenerateScriptOnly.Checked);
            chkRemoveOldIntMG.Enabled = !cmdPopulate.Enabled;
            cmdGenerateScripts.Enabled = SelectedM2M.GenerateScriptsOKtoRun(chkGenerateScriptOnly.Checked);
        }
        private void ClearAssessmentFields()
        {
            txtRowsInBaseMG.Text = string.Empty;
            txtRowsIntMG.Text = string.Empty;
            txtRowsIntDim.Text = string.Empty;
            txtRowsDim.Text = string.Empty;
            txtMaxLenManyKey.Text = string.Empty;
            txtMatrixKeyParts.Text = string.Empty;
            txtMaxAcceptableOnManySide.Text = string.Empty;
            txtNewRowCount.Text = string.Empty;
            txtCompressionRatio.Text = string.Empty;
            txtMatrixHashKeyLength.Text = string.Empty;
        }
        private void lvProgress_DoubleClick(object sender, EventArgs e)
        {
            DisplayProgressText(lvProgress);
        }
        private void DisplayProgressText(ListView lv)
        {
            if (lv.SelectedItems.Count > 0)
            {
                clsProgress prog=(clsProgress)lv.SelectedItems[0].Tag;
                StringBuilder sb = new StringBuilder();
                sb.AppendLine(prog.TimeStamp.ToLongTimeString());
                sb.AppendLine(prog.Message);
                sb.AppendLine(string.Empty);
                sb.AppendLine(prog.Details);
                sb.AppendLine(prog.ObjectName);
                DisplayText(sb.ToString());
            }
        }
        private void Display_textBoxText(object sender, EventArgs e)
        {
            if (sender is Control)
            {
                DisplayText(((Control)sender).Text);
            }
        }
        private void DisplayText(string text)
        {
            frmText f = new frmText(text);
            f.ShowDialog();
        }

        private void txtConnectionString_DoubleClick(object sender, EventArgs e)
        {
            DisplayText(txtConnectionString.Text);
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmAbout f = new frmAbout();
            f.ShowDialog();
        }
        /// <summary>
        /// Apply changes to both the underlying data source and the cube.
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdGenerateScripts_Click(object sender, EventArgs e)
        {
            /*
             * [12/19/2009] Eugene Asahara
             * I got an error with the SSAS transaction not able to commit. I think it was because I
             * had the database directly open in BIDS.
             * 
             * Also, there seemed to be a problem with the Data Mining models. I took them out out the
             * AdventureWorks SSAS database. I wasn't able to figure it out.
            */ 
            if (SelectedM2M == null) return;
            HourGlass h = new HourGlass();
            SelectedM2M.GenerateScripts(serv, selectedcube, chkGenerateScriptOnly.Checked,chkRemoveOldIntMG.Checked);
            DisplayMessageIfWarnings();
            SetButtons();
        }
        private clsM2M SelectedM2M
        {
            get
            {
                if (lvRelationships.SelectedItems.Count >0)
                {
                    return (clsM2M)lvRelationships.SelectedItems[0].Tag;
                }
                return null;
            }
        }

        private void txtMatrixTableName_Leave(object sender, EventArgs e)
        {
            SelectedM2M.MatrixTableName = txtMatrixTableName.Text;
        }
        /// <summary>
        /// Convert a column from the DSV into a DataItem object.
        /// </summary>
        /// <param name="dsv"></param>
        /// <param name="tableName"></param>
        /// <param name="columnName"></param>
        /// <returns></returns>
        private DataItem CreateDataItem(DataSourceView dsv, string tableName, string columnName)
        {
            DataTable dataTable = ((DataSourceView)dsv).Schema.Tables[tableName];
            DataColumn dataColumn = dataTable.Columns[columnName];
            return new DataItem(tableName, columnName, OleDbTypeConverter.GetRestrictedOleDbType(dataColumn.DataType));
        }

        private void cmdPopulate_Click(object sender, EventArgs e)
        {
            if (SelectedM2M == null) return;
            HourGlass h = new HourGlass();
            SelectedM2M.PopulateMatrix(chkGenerateScriptOnly.Checked);
            DisplayMessageIfWarnings();
            SetButtons();
            lvRelationships.SelectedItems[0].ImageIndex = (int)ProgressIcons.Done;
        }

        private void lstMaxItems_SelectedIndexChanged(object sender, EventArgs e)
        {
            /*
            if (lstMaxItems.SelectedItems.Count > 0 && SelectedM2M != null)
            {
                SelectedM2M.MaxItemsOnManySide = Convert.ToInt32(lstMaxItems.SelectedItems[0].SubItems[0].Text);
                txtMatrixHashKeyLength.Text = SelectedM2M.MatrixHashKeyLength.ToString();
            }
            */
        }

        private void lvProgress_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void tbClear_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure?","Clear the Progress Log.", MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                InitProgress();
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            DisconnectSSASServer();
            if (lvProgress.Items.Count > 0)
            {
                switch (MessageBox.Show("Save the Progress Report before closing? The progress report may contain useful code snippets (ex: for ETL).", "Closing the Application", MessageBoxButtons.YesNo, MessageBoxIcon.Warning))
                {
                    case DialogResult.Yes:
                        SaveProgress();
                        break;
                    //case DialogResult.Cancel:
                    //    e.Cancel = true;
                    //    break;
                }
            }
        }
        private void SaveProgress()
        {
            saveFileDialog1.ShowDialog();
        }

        private void cmdSaveLog_Click(object sender, EventArgs e)
        {
            SaveProgress();
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            WriteProgressLog();
        }
        private void WriteProgressLog()
        {
            HourGlass h = new HourGlass();
            StreamWriter sw = new StreamWriter(saveFileDialog1.FileName, false);
            int collcount;
            //TODO: Figure out how to cast SelectedItems and Items to a common class.
            //This will display items that are selected if any are selected or all if nothing is selected.
            if (lvProgress.SelectedItems.Count > 0)
            {
                collcount = lvProgress.SelectedItems.Count;
            }
            else
            {
                collcount = lvProgress.Items.Count;
            }
            for (int idx = 0; idx < collcount; idx++)
            {
                clsProgress prog;
                if (lvProgress.SelectedItems.Count > 0)
                {
                    prog = (clsProgress)lvProgress.SelectedItems[idx].Tag;
                }
                else
                {
                    prog = (clsProgress)lvProgress.Items[idx].Tag;
                }
                sw.WriteLine(prog.TimeStamp.ToLongTimeString());
                sw.WriteLine(prog.Message);
                sw.WriteLine(prog.Details);
                sw.WriteLine(string.Empty);
            }
            sw.Close();
            MessageBox.Show("Progress log successfully written to " + saveFileDialog1.FileName);
        }

        private void txtMatrixRealKeyFieldName_TextChanged(object sender, EventArgs e)
        {
            if (SelectedM2M != null)
            {
                SelectedM2M.MatrixRealKeyFieldName = txtMatrixRealKeyFieldName.Text;
            }
        }

        private void txtMatrixIntKeyName_TextChanged(object sender, EventArgs e)
        {
            if (SelectedM2M != null)
            {
                SelectedM2M.MatrixIntKeyName = txtMatrixIntKeyName.Text;
            }

        }

        private void txtMatrixTableName_TextChanged(object sender, EventArgs e)
        {
            if (SelectedM2M != null)
            {
                SelectedM2M.MatrixTableName = txtMatrixTableName.Text;
            }

        }

        private void tbAssessments_Enter(object sender, EventArgs e)
        {
        }

        private void cmdAssessment_Click(object sender, EventArgs e)
        {
            Assessment();
            cmdAssessment.Enabled = false;
        }

        private void lstRelationshipWarnings_DoubleClick(object sender, EventArgs e)
        {
            DisplayProgressText(lstRelationshipWarnings);
        }

        private void cmdTestConnection_Click(object sender, EventArgs e)
        {
            
        }

        private void chkGenerateScriptOnly_CheckedChanged(object sender, EventArgs e)
        {
            SetButtons();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void cmdDisconnect_Click(object sender, EventArgs e)
        {
            DisconnectSSASServer();
        }

        private void txtMaxAcceptableOnManySide_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (SelectedM2M != null)
                {
                    SelectedM2M.MaxItemsAcceptableOnManySide = Convert.ToInt32(txtMaxAcceptableOnManySide.Text);
                }
            }
            catch
            {
            }
        }

        private void lstMaxItems_DoubleClick(object sender, EventArgs e)
        {
            if (lstMaxItems.SelectedItems.Count > 0)
            {
                string txt = lstMaxItems.SelectedItems[0].SubItems[2].Text;
                DisplayText(txt);
            }
        }

        private void txtCommandTimeout_TextChanged(object sender, EventArgs e)
        {
        }

        private void txtCommandTimeout_Leave(object sender, EventArgs e)
        {
            try
            {
                clsM2M.CommandTimeout = Convert.ToInt32(txtCommandTimeout.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

        }

        private void txtConnectionString_Leave(object sender, EventArgs e)
        {
            clsM2M.ConnectionString = txtConnectionString.Text;
        }

        private void txtMaxLenManyKey_Leave(object sender, EventArgs e)
        {
            if (clsUtilities.isNumeric(txtMaxLenManyKey.Text))
            {
                SelectedM2M.MaxManyKeyLength = Convert.ToInt32(txtMaxLenManyKey.Text);
                cmdAssessment.Enabled = true;
            }
            else
            {
                MessageBox.Show("The max key length must be a value between 1 and 99. It is the number of characters in the max key value of the 'Many' dimension.");
            }
        }
    }
}