/*
 * Developed by Eugene A. Asahara, November-December 2007.
 * 
 * This application applies an algorithm to compress large many to many intermediate fact tables in Analysis Services cubes.
 * The algorithm is described in Erik Veermans blog: http://blogs.solidq.com/EN/Erik/Lists/Posts/Post.aspx?ID=1. 
 * It encapsulates a complex process and relatively complex T-SQL that wouldnt be of too much value to advanced SSAS folks 
 * applying this to a cube with just a couple of candidate many to many relationships. But perhaps there 
 * are many not-too-advanced SSAS folks out there tweaking cubes with a few or even a lot of such relationships.
 * 
/*============================================================================
  File:    clsM2M.cs

  Summary: Contains the logic around the Compression algorithm.

           Part of CompressManyToMany

  Date:    February 2008
------------------------------------------------------------------------------

  Copyright (C) Microsoft Corporation.  All rights reserved.

  This source code is intended only as a supplement to Microsoft
  Development Tools and/or on-line documentation.  See these other
  materials for detailed information regarding Microsoft code samples.

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.OleDb;
using Microsoft.AnalysisServices;

namespace CompressManyToMany
{

    /// <summary>
    /// This class holds information for a many to many relationship in a cube. These relationships
    /// are held in a ManyToManyMeasureGroupDimension object.
    /// </summary>
    internal class clsM2M
    {
        /*
         * General notes:
         * 
         * There are four tables involved in a many to many relationship:
         *      Measure group - This is the base measure upon which two dimensions relate to it in a series.
         *      Intermediate Dimension - This is the dimension directly connected to the Measure Group.
         *      Intermediate Measure Group - This table defines the many to many relationships between the intermediate dimension and the dimension.
         *      Dimension - 
         * 
         * Generally, variables beginning with "n" refer to the Intermediate Dimension, and variables beginning
         * with "m" refer to the dimension. This is as in "m:n".
        */
        static internal bool IsRealTable(ColumnBinding cb, DataSourceView dsv)
        {
            DataTable dt = dsv.Schema.Tables[cb.TableID];
            return dt.ExtendedProperties["TableType"].ToString() == "Table";
        }

        static internal string TableName(ColumnBinding cb, DataSourceView dsv)
        {
            DataTable dt = dsv.Schema.Tables[cb.TableID];
            string schema = string.Empty;
            if (dt.ExtendedProperties["DbSchemaName"] != null) schema="[" + dt.ExtendedProperties["DbSchemaName"].ToString() + "].";
            return schema+"["+dt.ExtendedProperties["DbTableName"].ToString() + "]";
        }
        static internal string FieldName(ColumnBinding cb, DataSourceView dsv)
        {
            return dsv.Schema.Tables[cb.TableID].Columns[cb.ColumnID].ExtendedProperties["DbColumnName"].ToString();
        }
        static internal bool FieldIsNumeric(ColumnBinding cb, DataSourceView dsv)
        {
            //What I really want to do is be sure the field is a numeric type so it can be summed.
            //TODO: The way I've done this currently doesn't prove for sure it's numeric, but close enough for now.
            DataColumn dc = dsv.Schema.Tables[cb.TableID].Columns[cb.ColumnID];
            return (dc.DataType.IsValueType);
        }
        static internal Type FieldType(ColumnBinding cb, DataSourceView dsv)
        {
            return dsv.Schema.Tables[cb.TableID].Columns[cb.ColumnID].DataType;
        }
        static internal string FullFieldName(ColumnBinding cb, DataSourceView dsv)
        {
            DataTable dt = dsv.Schema.Tables[cb.TableID];
            DataColumn dc = dt.Columns[cb.ColumnID];
            string schema = string.Empty;
            if (dt.ExtendedProperties["DbSchemaName"]!=null) schema="[" + dt.ExtendedProperties["DbSchemaName"].ToString() + "].";
            return schema+"[" + dt.ExtendedProperties["DbTableName"].ToString() + "].[" + dc.ExtendedProperties["DbColumnName"] + "]";
        }
        static internal string PlainTableName(ColumnBinding cb, DataSourceView dsv)
        {
            return dsv.Schema.Tables[cb.TableID].ExtendedProperties["DbTableName"].ToString();
        }
        static internal MeasureGroupAttribute MeasureGroupGranularityAttribute(RegularMeasureGroupDimension mgd)
        {
            foreach (MeasureGroupAttribute mga in mgd.Attributes)
            {
                if (mga.Type == MeasureGroupAttributeType.Granularity) return mga;
            }
            return null;
        }

        internal static readonly int UnsetValue = -1;

        public delegate void ProgressDelegate(object sender, ProgressEventArgs e);
        public event ProgressDelegate Progress;

        private const string TSQLComment = "--";
        private const string SpecialSQLTag = "--*** Useful query ***";
        private const string DefaultRelationalSchema = "dbo"; //Later, I'd like to let the user choose the schema for the new matrix table.
        private const double DigitsInBigInt = 18;   //The actual max value is 9223372036854775808, which is 19 digits. However,
        //it's possible to go over that value and result in an overflow. So, I've
        //made the limit 18 digits.

        private bool bProperlyInited = false;
        private int miTaskCount;            //Keeps track of how many "task" or "warning" messages are
                                            //created during an operation such as "Assessment". The UI
                                            //will display a message reminding the user to check the task list.
        internal int TaskCount
        {
            get { return miTaskCount; }
        }


        internal bool IntDimIsReal
        {
            get { return mbIntDimIsReal; }
        }
        private bool mbIntDimIsReal;  //We need to know that the intermediate dimension is real
                                                    //because we need to alter its schema. That is, add
                                                    //the matrix int key that it will join to.

        internal List<clsProgress> Errors = new List<clsProgress>();

        /// <summary>
        /// This is actually m2mIntDim.MeasureGroup.Name. However, the MeasureGroup reference is
        /// destroyed after application (GenerateScript is run), so we'll store this here for reference 
        /// after GenerateScript is fun.
        /// </summary>
        internal string OldIntDimName
        {
            get { return msOldIntDimName; }
        }
        private string msOldIntDimName;

        internal bool ProperlyInited
        {
            get { return bProperlyInited; }
        }
        /// <summary>
        /// This means cannot apply changes Assessment(), or populate the new objects.
        /// </summary>
        internal bool ShowStopper
        {
            get
            {
                //Is all flags that are showstoppers with ||.
                return !interDimToDimIsNumeric;
            }
        }
        //This is a show-stopper as far as actually populating the relational tables.
        //That's because this condition should prevent the matrix key hashing algorithm from working.
        private bool interDimToDimIsNumeric;

        private string m2mName;

        /// <summary>
        /// Allow setting of command timeout (defaulted to 30 in OleDbCommand). This deals with
        /// large databases, so it should be pretty big.
        /// </summary>
        public static int CommandTimeout
        {
            get { return miCommandTimeout; }
            set
            {
                if (value >= 0 && value < 99999) miCommandTimeout = value;
            }
        }
        private static int miCommandTimeout = 0;    //Set it to 0 for no timeout.

        /// <summary>
        /// The ability to override the ConnectionString of the Primary Data Source of the DSV
        /// will allow users to override certain parameters.
        /// </summary>
        public static string ConnectionString
        {
            get { return msConnectionString; }
            set
            {
                msConnectionString = value;
            }
        }
        private static string msConnectionString=string.Empty;    //Set it to 0 for no timeout.

        //This should be the only constructor. Other methods assume BaseMG, m2nMGDim, and IntermediateDimension
        //are assigned values.
        internal clsM2M(MeasureGroup BaseMG,ManyToManyMeasureGroupDimension m2nMGDim, MeasureGroupDimension IntermediateDimension,DataSourceView dsv)
        {
            this.dsv = dsv;
            //clsM2M.ConnectionString = dsv.DataSource.ConnectionString;
            //
            baseMG = BaseMG;
            this.m2nMGDim = m2nMGDim;
            msOldIntDimName = this.m2nMGDim.MeasureGroup.Name;
            intermediateDimension = IntermediateDimension;
            mbChangesApplied = this.m2nMGDim.Annotations.Contains(ChangesAppliedExtendedProperty);
            //m2mName is what ToString() returns. After changes are applied, the name should be different,
            //but leave it this way to show what it was.
            StringBuilder sb = new StringBuilder();
            sb.Append(BaseMG.Name + "-->" + IntermediateDimension.Dimension.Name + "-->");
            sb.Append(Many2ManyMG.Name + "-->" + m2nMGDim.Dimension.Name);
            m2mName=sb.ToString();

        }
        /// <summary>
        /// Initializes the class after the constructor is called.
        /// </summary>
        internal bool InitClass()
        {
            if (bProperlyInited) return false;        //This class should be called only once.
                                                      //I would have made this method part of the constructor,
                                                //but it sends Progress events. However, the delegate to the class
                                                //isn't set up in the constructor.
            //
            if (mbChangesApplied)
            {
                AddProgress("The compression algorithm is already applied to this relationship by this application.", string.Empty, this.ToString(), ProgressIcons.Informational);
            }
            try
            {
                matrixTableName = Many2ManyMG.Name + "_Matrix"; //Set a default name for the matrix lookup table.
                //
                RegularMeasureGroupDimension regMGDim;
                MeasureGroupAttribute granattr;
                ColumnBinding cb = null;           //A general variable so we don't repeat some ugly object path.
                //Fill fields for Measure Group --> Intermediate Dimension.
                //Be sure it's a class derived from RegularMeasureGroupDimension; ex: DegenerateMeasureGroupDimension.
                if (!(BaseMG.Dimensions[IntermediateDimension.CubeDimensionID] is RegularMeasureGroupDimension))
                {
                    //TODO: This has to be taken care of. I think this has to do with cascading many to many.
                    //Unable to raise a Progress event since the delegate hasn't yet been declared.
                    AddProgress("This relationship is of a wrong type: " +
                        BaseMG.Dimensions[IntermediateDimension.CubeDimensionID].GetType().Name,
                        "This application currently cannot handle cascading m:n relationships.", this.ToString(),
                        ProgressIcons.Warning);
                    return false;
                }
                regMGDim = ((RegularMeasureGroupDimension)BaseMG.Dimensions[IntermediateDimension.CubeDimensionID]);
                granattr = clsM2M.MeasureGroupGranularityAttribute(regMGDim);
                if (granattr != null && granattr.KeyColumns.Count > 0)
                {
                    LinkToBaseMG = new List<string>();
                    DimLinkColumn = new List<string>();
                    for (int idx = 0; idx < granattr.KeyColumns.Count; idx++)
                    {
                        if (idx == 0)
                        {
                            msTableName[(int)Tables.BaseMeasureGroup] =
                                clsM2M.TableName(((ColumnBinding)granattr.KeyColumns[idx].Source), dsv);
                        }
                        LinkToBaseMG.Add(clsM2M.FullFieldName(((ColumnBinding)granattr.KeyColumns[idx].Source), dsv));
                        DimLinkColumn.Add(clsM2M.FullFieldName(((ColumnBinding)regMGDim.Dimension.Attributes[granattr.AttributeID].KeyColumns[idx].Source), dsv));
                    }
                }
                //Fill fields for Intermediate Measure Group --> Intermediate Dimension.
                regMGDim = ((RegularMeasureGroupDimension)Many2ManyMG.Dimensions[IntermediateDimension.CubeDimensionID]);
                granattr = clsM2M.MeasureGroupGranularityAttribute(regMGDim);
                if (granattr != null && granattr.KeyColumns.Count > 0)
                {
                    IM2IDLinkToMG = new List<string>();
                    IM2IDLinkToDim = new List<string>();
                    for (int idx = 0; idx < granattr.KeyColumns.Count; idx++)
                    {
                        IM2IDLinkToMG.Add(clsM2M.FullFieldName(((ColumnBinding)granattr.KeyColumns[idx].Source), dsv));
                        cb = ((ColumnBinding)regMGDim.Dimension.Attributes[granattr.AttributeID].KeyColumns[idx].Source);
                        IM2IDLinkToDim.Add(clsM2M.FullFieldName(cb, dsv));
                        if (idx == 0)
                        {
                            msTableName[(int)Tables.IntermediateDimension] = clsM2M.TableName(cb, dsv);
                        }
                    }
                    //The intermediate dimension table will have the matrix key added to it. So, if it's not a real table, this
                    //change cannot be made.
                    mbIntDimIsReal = IsRealTable(cb, dsv);
                    if (!mbIntDimIsReal)
                    {
                        AddProgress("The intermediate dimension table is not a real table.", "The matrix key cannot be added to this table.", msTableName[(int)Tables.IntermediateDimension], ProgressIcons.Warning);
                    }
                    cb = (ColumnBinding)regMGDim.Dimension.Attributes[granattr.AttributeID].KeyColumns[0].Source;
                    interDimTablePlainName = clsM2M.PlainTableName(cb, dsv);
                    interdimTableID = cb.TableID;
                }
                //Fill fields for Intermediate Measure Group --> Dimension.
                if (!(Many2ManyMG.Dimensions[CubeDimension.ID] is RegularMeasureGroupDimension))
                {
                    //Non-regular relationships (Referenced) create more complexity that isn't handled at this time.
                    //TODO: Take care of m2m relationships where the relationshipship includes a refernced dim.
                    AddProgress("Cannot handle non-regular relationships between the Many Dimension and the Intermediate Measure Group at this time.", "Relation type: " + (Many2ManyMG.Dimensions[CubeDimension.ID]).GetType().ToString(), this.ToString(), ProgressIcons.Informational);
                    return false;
                }
                regMGDim = ((RegularMeasureGroupDimension)Many2ManyMG.Dimensions[CubeDimension.ID]);
                granattr = clsM2M.MeasureGroupGranularityAttribute(regMGDim);
                if (granattr != null && granattr.KeyColumns.Count > 0)
                {
                    if (granattr.KeyColumns.Count > 1)
                    {
                        //Unlike the other two keys (baseMG-->InterDim and InterDim-->InterMG), this key cannot
                        //theoretically be composite due to the matrix hash algorithm used right now. It must
                        //be a numeric type.
                        AddProgress("The key linking the intermediate MG and the dimension is composite. Only the first part of the key will be considered.", KeyColumnsString(granattr.KeyColumns), string.Empty, ProgressIcons.Warning);
                    }
                    //If there is a referenced relationship, the Source could be of type InheritedBinding,
                    //which triggers an invalid cast.
                    //TODO: This is a big pain and needs to be handled later.
                    cb = (ColumnBinding)granattr.KeyColumns[0].Source;
                    interDimToDimIsNumeric = FieldIsNumeric(cb, dsv);
                    if (!interDimToDimIsNumeric)
                    {
                        //This key must be numeric since the algorithm to create the matrix hash key sums the
                        //values of the keys on the many side to form the hash of all the keys on the many side.
                        //This is a show-stopper.
                        AddProgress("The key linking the intermediate MG (" + msTableName[(int)Tables.IntermediateDimension] + ") and the dimension (" + clsM2M.TableName(cb, dsv) + ") must be numeric. This is a show-stopper.", "The type is: " + FieldType(cb, dsv).ToString() + "; the columns are " + KeyColumnsString(granattr.KeyColumns), this.ToString(), ProgressIcons.Error);
                    }
                    IM2DToMG = clsM2M.FullFieldName(cb, dsv);
                    matrixRealKeyFieldName = clsM2M.FieldName(cb, dsv) + "_MatrixHash";
                    matrixIntKeyName = clsM2M.FieldName(cb, dsv) + "_MatrixIntKey";
                    cb = (ColumnBinding)regMGDim.Dimension.Attributes[granattr.AttributeID].KeyColumns[0].Source;
                    IMGDim = clsM2M.FullFieldName(cb, dsv);
                    dimKeyName = clsM2M.FieldName(cb, dsv);
                    dimTableID = cb.TableID;
                    //Using TableName[] instead.
                    msTableName[(int)Tables.ManyDimension] = clsM2M.TableName(cb, dsv);
                }
                bProperlyInited = true;
                return true;
            }
            catch (Exception ex)
            {
                AddProgress("Error in InitClass(). Skipping this m2m relationship.",ex.Message,this.ToString(), ProgressIcons.Error);
                bProperlyInited = false;
                return false;
            } 
        }

        private string KeyColumnsString(DataItemCollection dic)
        {
            StringBuilder sb = new StringBuilder();
            foreach (DataItem di in dic)
            {
                sb.Append(clsM2M.FullFieldName((ColumnBinding)di.Source, dsv) + "/");
            }
            return sb.ToString();
        }

        internal string MatrixRealKeyFieldName
        {
            get { return matrixRealKeyFieldName; }
            set { matrixRealKeyFieldName = value; }
        }
        private string matrixRealKeyFieldName;


        internal string MatrixIntKeyName
        {
            get { return matrixIntKeyName; }
            set { matrixIntKeyName = value; }
        }
        private string matrixIntKeyName;
        
        internal string MatrixTableName
        {
            get { return matrixTableName; }
            set { matrixTableName = value; }
        }
        private string matrixTableName;

        
        internal string IM2DToMG;
        internal string IMGDim;
        internal List<string> IM2IDLinkToMG;
        internal List<string> IM2IDLinkToDim;
        internal List<string> LinkToBaseMG;         //List of Key Columns database names of the Base Measure Group.
        internal List<string> DimLinkColumn;

        private DataSourceView dsv;

        /// <summary>
        /// This is an important constant. This is an important value. If a member is related to very many
        /// members (ex: a sale is attributed to 2000 campaigns), this method will result in a huge real matrix
        /// key (hash) that will overflow the length of the sql that generates the keys. It's possible that
        /// there is a better way.
        /// 
        /// It's important to note too that if there are members related to lots of members in a m:n relationship,
        /// it's likely they are all different combinations, which means the new matrix lookup table will be
        /// not much shorter than the current intermediate fact table.
        /// 
        /// Example:
        /// 
        /// There are many search words for a web visit. There could be one search word or dozens. There
        /// would be so many combinations of search words that the intermediate measure group would be huge.
        /// If miMaxItemsAcceptableOnManySide=10, this many to many relationship would be rejected because
        /// the max number of search words per web visit would be far greater than 10.
        /// </summary>
        public int MaxItemsAcceptableOnManySide
        {
            get { return miMaxItemsAcceptableOnManySide; }
            set 
            {
                if (value > 0 && value < 999)
                {
                    miMaxItemsAcceptableOnManySide = value;
                }
            }
        }
        private int miMaxItemsAcceptableOnManySide = 10;

        internal int MaxItemsOnManySide
        {
            get { return miMaxItemsOnManySide; }
            set
            {
                miMaxItemsOnManySide = value;
                assessmentDone = false;
                Assessment();
            }
        }
        private int miMaxItemsOnManySide;


        /// <summary>
        /// This is the object that contains information linking the M2MMG to the M2M Dimension.
        /// </summary>
        internal ManyToManyMeasureGroupDimension M2M
        {
            get { return m2nMGDim; }
            
        }
        private ManyToManyMeasureGroupDimension m2nMGDim;
        /// <summary>
        /// The Base Measure Group are the actual facts. This is as opposed to the Intermediate Measure Group
        /// which is an association table associated many entities to one entity (ex: Customers to Account).
        /// </summary>
        internal MeasureGroup BaseMG
        {
            get { return baseMG; }
        }
        private MeasureGroup baseMG;

        internal MeasureGroupDimension IntermediateDimension
        {
            get { return intermediateDimension; }
        }
        private MeasureGroupDimension intermediateDimension;

        private int maxManyKeyLength = 0;   //This initial value of zero should trigger a SQL to get the
                                            //actual value in the Assessment routine.

        internal int MaxManyKeyLength
        {
            get { return maxManyKeyLength; }
            set
            {
                //A max value of two means that we're theoretically allowing a 99 digit key.
                //This should be way enough.
                //We will allow a value of 0 which will trigger an automatic fetch of the value in Assessment().
                if (maxManyKeyLength >= 0 && maxManyKeyLength <= 99)
                {
                    maxManyKeyLength = value;
                }
            }
        }

        internal int RowsInNewIntermediateMG
        {
            get { return rowsInNewIntermediateMG; }
        }
        private int rowsInNewIntermediateMG;


        internal int MatrixHashKeyMaxLength
        {
            get { return MaxItemsOnManySide*MaxManyKeyLength; }
        }

        private List<string> nfieldname;    //List of the keys in the intermediate dimension.
        private List<string> mplainfieldname;   //List of Many dimension field without brackets.
        private string mplaintablename = string.Empty;
        private List<string> mfieldname;
        private string matrixtablesql = string.Empty;
        private string mgSQL = string.Empty;
        private const string mgTempName = "#mg";

        private string matrixkeysql = string.Empty;
        private string setmatrixkeysql = string.Empty;

        internal string TableName(Tables tabletype)
        {
            return msTableName[(int)tabletype];
        }
        private string[] msTableName=new string[4];

        internal long TableRows(Tables tabletype)
        {
            return miTableRows[(long)tabletype];
        }
        private long[] miTableRows = new long[4];

        private string interDimTablePlainName;
        private string interdimTableID;
        private string dimTableID;
        private string dimKeyName;
        internal List<RankCount> RankCounts;

        private int numberofmatrixkeyparts;

        public int MatrixHashKeyLength
        {
            get { return Convert.ToInt32(numberofmatrixkeyparts * DigitsInBigInt); }
        }

        internal RegularMeasureGroupDimension M2Mmg_to_InterDim
        {
            get
            {
                return ((RegularMeasureGroupDimension)Many2ManyMG.Dimensions[IntermediateDimension.CubeDimensionID]);
            }
        }

        internal double CompressionRatio
        {
            get
            {
                if (miTableRows[(int)Tables.IntermediateMeasureGroup] != 0)
                {
                    return (100.0 * ((Convert.ToDouble(miTableRows[(int)Tables.IntermediateMeasureGroup]) - RowsInNewIntermediateMG) / miTableRows[(int)Tables.IntermediateMeasureGroup]));
                }
                return 0;
            }
        }

        internal MeasureGroup Many2ManyMG
        {
            get 
            {
                //TODO: After applying, m2nMGDim.MeasureGroup is is different. Handling this properly
                //needs to be done later.
                try
                {
                    return m2nMGDim.MeasureGroup;
                }
                catch
                {
                    return null;
                }
            }
        }
        internal CubeDimension CubeDimension
        {
            get { return m2nMGDim.CubeDimension; }
        }

        private bool assessmentDone = false;
        internal bool AssessmentDone
        {
            get { return assessmentDone; }
        }

        internal bool AcceptableMaxItemsOnManySide
        {
            get { return MaxItemsOnManySide <= miMaxItemsAcceptableOnManySide; }
        }
        public override string ToString()
        {
            return m2mName;
        }
        internal bool AssessmentOKtoRun
        {
            get
            {
                //Can run the assessment as long as the changes haven't been applied.
                return !mbChangesApplied && !ShowStopper;
            }
        }

        internal int MatrixKeyParts
        {
            get { return numberofmatrixkeyparts; }
        }
        internal void ResetAssessmentValues()
        {
            MaxManyKeyLength = 0;
        }

        internal void Assessment()
        {
            InitOperation();
            if (!AssessmentOKtoRun) return;
            //
            OleDbCommand scmd;
            OleDbConnection sconn=null;
            RegularMeasureGroupDimension regMGDim;
            List<string> mfullfieldname=new List<string>();         //Name of the field on the "many" side of the relationship.
            msTableName[(int)Tables.IntermediateMeasureGroup] = string.Empty;
            List<string> nfullfieldname=new List<string>();
            string nfulltablename = string.Empty;
            MeasureGroupAttribute granattr;
            //
            try
            {
                sconn = OpenDataSource();
                if (sconn == null) return;
                //
                granattr = clsM2M.MeasureGroupGranularityAttribute(M2Mmg_to_InterDim);
                nfieldname = new List<string>();
                if (granattr != null && granattr.KeyColumns.Count > 0)
                {
                    SetFullFieldName(granattr.KeyColumns, ref nfullfieldname, ref nfieldname, ref nfulltablename);
                }
                regMGDim = ((RegularMeasureGroupDimension)Many2ManyMG.Dimensions[CubeDimension.ID]);
                granattr = clsM2M.MeasureGroupGranularityAttribute(regMGDim);
                mfieldname = new List<string>();
                mplainfieldname = new List<string>();
                if (granattr != null && granattr.KeyColumns.Count > 0)
                {
                    SetFullFieldName(granattr.KeyColumns, ref mfullfieldname, ref mfieldname, ref msTableName[(int)Tables.IntermediateMeasureGroup]);
                    for (int idx = 0; idx < granattr.KeyColumns.Count; idx++)
                    {
                        mplainfieldname.Add(clsM2M.FieldName(((ColumnBinding)granattr.KeyColumns[idx].Source), dsv));
                    }
                    mplaintablename = clsM2M.PlainTableName(((ColumnBinding)granattr.KeyColumns[0].Source), dsv);
                    //Already done in SetFullFieldName.
                    //msTableName[(int)Tables.ManyDimension] = clsM2M.TableName(((ColumnBinding)granattr.KeyColumns[0].Source), dsv);
                }
                //
                granattr = clsM2M.MeasureGroupGranularityAttribute(M2Mmg_to_InterDim);
                if (granattr != null && granattr.KeyColumns.Count > 0)
                {
                    StringBuilder sql = new StringBuilder();
                    msTableName[(int)Tables.IntermediateMeasureGroup] = clsM2M.TableName(((ColumnBinding)granattr.KeyColumns[0].Source), dsv);
                    //[Eugene Asahara] Modified to cast Count(*) as bigint.
                    sql.AppendLine("SELECT 'BaseMG',COUNT_BIG(*) FROM " + msTableName[(int)Tables.BaseMeasureGroup]);
                    sql.AppendLine("SELECT 'IntDim',COUNT_BIG(*) FROM " + msTableName[(int)Tables.IntermediateDimension]);
                    sql.AppendLine("SELECT 'IntMG',COUNT_BIG(*) FROM " + msTableName[(int)Tables.IntermediateMeasureGroup]);
                    sql.AppendLine("SELECT 'Dim',COUNT_BIG(*) FROM " + msTableName[(int)Tables.ManyDimension]);
                    //
                    scmd =OpenCommand(sql.ToString(), sconn);
                    AddProgress("Getting row count of tables of concern.", sql.ToString(), msTableName[(int)Tables.IntermediateMeasureGroup], ProgressIcons.SQL);
                    OleDbDataReader dr = scmd.ExecuteReader();
                    //I chose to do this loop instead of a single for loop because I ran into a problem
                    //with missing tables.
                    do
                    {
                        while (dr.Read())
                        {
                            long val = Convert.ToInt64(dr[1]);
                            switch (dr[0].ToString())
                            {
                                case "BaseMG":
                                    miTableRows[(long)Tables.BaseMeasureGroup] = val;
                                    break;
                                case "IntMG":
                                    miTableRows[(long)Tables.IntermediateMeasureGroup] = val;
                                    break;
                                case "IntDim":
                                    miTableRows[(long)Tables.IntermediateDimension] = val;
                                    break;
                                case "Dim":
                                    miTableRows[(long)Tables.ManyDimension] = val;
                                    break;
                            }
                        }
                    } while (dr.NextResult());
                    dr.Close();
                    //
                    //Create query to list the many to many relationships.
                    sql = new StringBuilder();
                    sql.Append("SELECT " +clsM2M.FormatStringArray( nfieldname,",") + "," + clsM2M.FormatStringArray(mfieldname,",") + ", ROW_NUMBER() OVER (PARTITION BY " + clsM2M.FormatStringArray(nfieldname,",") + " ORDER BY " + clsM2M.FormatStringArray(mfieldname,",") + ") Rank ");
                    sql.Append("into " + mgTempName + " FROM " + msTableName[(int)Tables.IntermediateMeasureGroup] + " ");
                    sql.Append("GROUP BY  " + clsM2M.FormatStringArray(nfieldname,",") + "," + clsM2M.FormatStringArray(mfieldname,","));
                    mgSQL = sql.ToString();     //Save this for use when populating. #mg will need to be created in that session.
                    scmd.CommandText = mgSQL;
                    scmd.ExecuteNonQuery();
                    AddProgressSQL("SQL to list many to many relationships.", new string[] 
                    {
                        "This creates temp table holding a list of the found 'many' patterns that is used in several queries."
                    }, mgSQL, mgTempName);
                    //
                    sql = new StringBuilder();
                    sql.Append("SELECT Rank,count(Rank) FROM "+mgTempName+" AS SUB ");
                    sql.Append("GROUP BY Rank ORDER BY Rank DESC");
                    scmd.CommandText = sql.ToString();
                    AddProgressSQL("Getting distribution of many to one relationships.", new string[] 
                    {
                        "Get distribution of many to one relationships."
                    }, sql.ToString(), mgTempName);
                    OleDbDataReader odr = scmd.ExecuteReader();
                    RankCounts = new List<RankCount>();
                    while (odr.Read())
                    {
                        RankCounts.Add(new RankCount(Convert.ToInt32(odr[0]), Convert.ToInt32(odr[1])));
                    }
                    odr.Close();
                    if (RankCounts.Count > 0)
                    {
                        //Notice I set the private version of miMaxItemsOnManySide. Setting the public
                        //version will call Assessment() - which would be recursive.
                        miMaxItemsOnManySide = RankCounts[0].Rank;
                    }
                    //
                    sql = new StringBuilder();
                    sql.Append("SELECT max(" + mfieldname[0] + ") FROM " + TableName(Tables.ManyDimension));
                    scmd.CommandText = sql.ToString();

                    if (MaxManyKeyLength == 0)
                    {
                        AddProgress("Getting maximum length of the 'Many' key.", scmd.CommandText, string.Empty, ProgressIcons.SQL);
                        object maxKeyValue = scmd.ExecuteScalar();
                        if (maxKeyValue is System.DBNull)
                        {
                            throw (new Exception("The are no values in the 'Many' dimension table from which to calculate the maximum length of the Many Key."));
                        }
                        MaxManyKeyLength = maxKeyValue.ToString().Length;
                    }
                    if (miTableRows[(int)Tables.IntermediateMeasureGroup] == 0)
                    {
                        //If there are no rows in the intermediate MG (=0), we will be unable to calculate
                        //the variable, numberofmatrixkeyparts.
                        throw (new Exception("The are no rows in the intermediate MG, "+msTableName[(int)Tables.IntermediateMeasureGroup]+"."));
                    }
                    numberofmatrixkeyparts = Convert.ToInt32(Math.Ceiling(MatrixHashKeyMaxLength / DigitsInBigInt));
                    //Any query from here on should not be run if the acceptable number of items on the many side is too high.
                    if (!AcceptableMaxItemsOnManySide)
                    {
                        //This means the max items on the many side exceeds the acceptable value.
                        //Queries from here until the comment, "End of MaxItemsOnManySide Alert", is reached.
                        AddProgress("The currently selected max items on the many side (" + MaxItemsOnManySide.ToString() + ") is greater than the currently acceptable number (" + miMaxItemsAcceptableOnManySide.ToString() + ").", "Queries for matrix keys will be generated, but not run.", string.Empty, ProgressIcons.Warning);
                    }
                    //
                    //This query will determine the matrix key for each member of the "n" dimension.
                    //It's used later to create the actual new IntermediateDimension.
                    //
                    //
                    sql = new StringBuilder();
                    //
                    //
                    //*** Start of code to build matrix keys
                    //
                    //The general idea of building the unique matrix keys are to create a string
                    //that is the concatenation of all keys of the many dimension.
                    //For example, if a sale is attributed to reasons 1, 3 and 5, that
                    //matrix key will be 050301. Notice the key is in reverse order? This is just 
                    //a fluke that I hope to fix later.
                    //
                    //The algorithm uses a sum to get this. Therefore, the hash key is created in parts
                    //that can has fewer digits than in a bigint. The keys are SUMed up. In the example of
                    //1, 3, and 5, the matrix key is (5*10000)+(3*100)+(1*1).
                    //The nature of this algorithm is why the key of the Many Dimension needs to be int.
                    int keysperpart = Convert.ToInt32(Math.Floor(DigitsInBigInt / MaxManyKeyLength));
                    StringBuilder conv = new StringBuilder();
                    StringBuilder conv1 = new StringBuilder();
                    for (int x1 = 1;x1<=numberofmatrixkeyparts; x1++)
                    {
                        conv.Append("CONVERT(VARCHAR,MatrixKey" + x1.ToString() + ")");
                        if (x1 < numberofmatrixkeyparts)
                        {
                            conv.Append("+");
                        }
                        else
                        {
                            conv.Append(" AS MatrixKey");
                        }
                    }
                    int matrixkeypartcount = 1;
                    StringBuilder keytemplate = new StringBuilder(new string(Convert.ToChar("0"), Convert.ToInt32(DigitsInBigInt)));
                    keytemplate[MaxManyKeyLength - 1] = Convert.ToChar("1");
                    for (int x1 = 1; x1 <= MaxItemsOnManySide; x1 += keysperpart)
                    {
                        int endkey = x1 + keysperpart;
                        conv1.Append("SUM(CASE WHEN SUB.[Rank] >=" + x1.ToString() + " AND SUB.[Rank] <" +
                            endkey.ToString() + " THEN SUB." + mfieldname[0] +
                            "* Convert(bigint,LEFT('" + keytemplate.ToString() +
                            "',((SUB.[Rank] - " + (x1 - 1).ToString() + ") * " +
                            MaxManyKeyLength.ToString() + "))) ELSE 0 END) MatrixKey" +
                            matrixkeypartcount.ToString());
                        if (x1 + keysperpart <= MaxItemsOnManySide) conv1.Append(",");
                        matrixkeypartcount++;
                    }
                    keytemplate = null;
                    //
                    sql.Append("SELECT " + clsM2M.FormatStringArray(nfieldname,",") +
                        "," + conv.ToString() + " into #matrixkey FROM (");
                    sql.Append("SELECT " + clsM2M.FormatStringArray(nfieldname,",") + "," + conv1.ToString());
                    sql.Append(" FROM " + mgTempName + " AS SUB GROUP BY " +
                        clsM2M.FormatStringArray(nfieldname,",") + ") AS A");
                    setmatrixkeysql = sql.ToString();
                    AddProgress("Built query to generate matrix keys.", sql.ToString(), string.Empty, ProgressIcons.SQL);
                    conv = null;
                    conv1 = null;
                    //*** End of code to build matrix keys
                    //
                    sql = new StringBuilder();
                    sql.Append("SELECT DISTINCT MATRIX.MatrixKey,FACT." + mfieldname[0] +
                        " FROM " + msTableName[(int)Tables.IntermediateMeasureGroup] + " FACT ");
                    sql.Append("INNER JOIN #matrixkey MATRIX ON ");
                    for (int idx = 0; idx < nfieldname.Count; idx++)
                    {
                        sql.Append("FACT."+ nfieldname[idx] + " = MATRIX." + nfieldname[idx]);
                        if (idx < nfieldname.Count - 1) sql.Append(" AND ");
                    }
                    matrixtablesql = sql.ToString();
                    AddProgress("Built query to populate the matrix table.", sql.ToString(), "matrixtablesql", ProgressIcons.SQL);
                    //
                    sql = new StringBuilder();
                    sql.Append("SELECT DISTINCT " + clsM2M.FormatStringArray("FACT.",nfieldname,",") +
                        ",MATRIX.MatrixKey FROM " + msTableName[(int)Tables.IntermediateMeasureGroup] + " FACT ");
                    sql.Append("INNER JOIN #matrixkey MATRIX ON ");
                    for (int idx = 0; idx < nfieldname.Count; idx++)
                    {
                        sql.Append("FACT." + nfieldname[idx] + " = MATRIX." + nfieldname[idx]);
                        if (idx < nfieldname.Count - 1) sql.Append(" AND ");
                    }
                    matrixkeysql = sql.ToString();
                    AddProgress("Built query to determine the matrix keys for the intermediate dimension.", sql.ToString(), "matrixkeysql", ProgressIcons.SQL);
                    //
                    if (AcceptableMaxItemsOnManySide)
                    {
                        scmd.CommandText=setmatrixkeysql;
                        scmd.ExecuteNonQuery();
                    }
                    sql = new StringBuilder();
                    scmd.CommandText = "SELECT count_big(*) from (" + matrixtablesql + ") as GetCount";
  
                    AddProgress("Getting row count for the new intermediate fact table.", scmd.CommandText, string.Empty, ProgressIcons.SQL);
                    if (AcceptableMaxItemsOnManySide)
                    {
                        rowsInNewIntermediateMG = Convert.ToInt32(scmd.ExecuteScalar());
                        CleanUpSQLSession(scmd);
                    }
                    else
                    {
                        rowsInNewIntermediateMG = UnsetValue;
                    }
                    //Clean up.
                    //!!!End of MaxItemsOnManySide Alert!!!
                    //
                    assessmentDone = true;      //Setting this flag enables the ability to make the schema changes.
                    //
                }
            }
        
            catch (Exception ex)
            {
                AddProgress("Error during assessment of the many to many relationship.", ex.Message, "cmdGenerate_Scripts_Click", ProgressIcons.Error);
                assessmentDone = false;
            }
            finally
            {
                CloseDataSourceConnection(sconn);
            }
         
        }
        private void CleanUpSQLSession(OleDbCommand cmd)
        {
            cmd.CommandText = "DROP TABLE " + mgTempName;
            cmd.ExecuteNonQuery();
            if (AcceptableMaxItemsOnManySide)
            {
                cmd.CommandText = "DROP TABLE " + "#matrixkey";
                cmd.ExecuteNonQuery();
            }
        }
        private void SetFullFieldName(DataItemCollection keycolumns, ref List<string> fullfieldname, ref List<string> fieldname,ref string fulltablename)
        {
            for (int idx = 0; idx < keycolumns.Count;idx++ )
            {
                ColumnBinding cb = (ColumnBinding)keycolumns[idx].Source ;
                DataTable dt = dsv.Schema.Tables[cb.TableID];
                if (dt.ExtendedProperties["TableType"].ToString() != "Table")
                {
                    throw (new Exception("The underlying table," + dt.ExtendedProperties["DbTableName"].ToString() + ", is a " + dt.ExtendedProperties["TableType"].ToString() + ", must have a TableType of 'Table'."));
                }
                DataColumn dc = dt.Columns[cb.ColumnID];
                fulltablename = "[" + dt.ExtendedProperties["DbSchemaName"].ToString() + "].[" + dt.ExtendedProperties["DbTableName"].ToString() + "]";
                fullfieldname.Add(fulltablename + ".[" + dc.ExtendedProperties["DbColumnName"] + "]");
                fieldname.Add("[" + dc.ExtendedProperties["DbColumnName"] + "]");
            }
        }

        private OleDbCommand OpenCommand(string sql, OleDbConnection conn)
        {
            OleDbCommand scmd = new OleDbCommand(sql.ToString(), conn);
            scmd.CommandTimeout = miCommandTimeout;
            return scmd;
        }

        private OleDbConnection OpenDataSource()
        {
            try
            {
                AddProgress("Opening connection to the relational database.", msConnectionString, dsv.DataSource.Name);
                OleDbConnection sconn = new OleDbConnection(msConnectionString);
                sconn.Open();
                return sconn;
            }
            catch (Exception ex)
            {
                AddProgress("Error attempting to open connection to the relational database.", ex.Message, "OpenDataSource", ProgressIcons.Error);
                return null;
            }
        }
        private void CloseDataSourceConnection(OleDbConnection conn)
        {
            if (dsv != null)
            {
                AddProgress("Closing connection to the data source.", clsM2M.ConnectionString, dsv.DataSource.Name);
            }
            if (conn != null && conn.State == ConnectionState.Open)
            {
                conn.Close();
            }
            conn = null;
        }
        internal bool PopulateMatrixOKtoRun(bool verbose,bool bGenerateScriptOnly)
        {
            bool oktorun = true;
            if (ShowStopper)
            {
                oktorun = false;
            }
            if (!bGenerateScriptOnly && !assessmentDone)
            {
                if (verbose)
                {
                    AddProgress("Assessment must be performed before populating the tables.", "Operation canceled.", "PopulateMatrix", ProgressIcons.Informational);
                }
                oktorun=false;
            }
            //It's OK to generate script before changes are applied. That will remain true as long
            //as this method isn't getting new schema information from the data source database.
            if (!bGenerateScriptOnly && !mbChangesApplied)
            {
                if (verbose)
                {
                    AddProgress("Schema changes must be applied before populating the tables.", "Operation canceled.", "PopulateMatrix", ProgressIcons.Informational);
                }
                oktorun = false;
            }
            return oktorun;
        }
        /// <summary>
        /// Populates the new matrix lookup table and the new field for the matrix int key in the intermediate dimension table.
        /// </summary>
        /// <param name="bGenerateScriptOnly"></param>
        /// <returns></returns>
        internal bool PopulateMatrix(bool bGenerateScriptOnly)
        {
            InitOperation();
            if (!PopulateMatrixOKtoRun(true, bGenerateScriptOnly)) return false;

            StringBuilder sql;
            OleDbConnection sconn=null;
            OleDbTransaction strans=null;
            OleDbCommand scmd=null;
            try
            {
                if (!bGenerateScriptOnly)
                {
                    AddProgress("Beginning tasks to populate the matrix objects.", string.Empty, "cmdPopulate_Click");
                    sconn = OpenDataSource();
                    if (sconn == null) return false;
                    strans = sconn.BeginTransaction();
                    scmd = new OleDbCommand();
                    scmd.Connection = sconn;
                    scmd.Transaction = strans;
                    scmd.CommandTimeout = miCommandTimeout;
                }
                //Before executing the population queries, we need to create a temp table.
                //Show the progress line anyway so that the user gets the SQL to create #mg and #matrixKey.
                //These are useful templates for ETL.
                AddProgressSQL("Temp table (#mg) used by the populating queries.", new string[] 
                    {
                        "Temp table (#mg) used by the populating queries."
                    }, mgSQL, MatrixTableName);
                if (!bGenerateScriptOnly)
                {
                    scmd.CommandText = mgSQL;
                    scmd.ExecuteNonQuery();
                }
                AddProgressSQL("Temp table (#matrixKey) used by the populating queries.", new string[] 
                    {
                        "Temp table (#matrixKey) used by the populating queries."
                    }, setmatrixkeysql, MatrixTableName);
                if (!bGenerateScriptOnly)
                {
                    scmd.CommandText = setmatrixkeysql;
                    scmd.ExecuteNonQuery();
                }
                //
                //Populate the new matrix lookup table.
                sql = new StringBuilder();
                sql.Append("INSERT INTO [" + MatrixTableName + "] ([" + MatrixRealKeyFieldName + "]," + mfieldname[0] + ",[" + MatrixIntKeyName + "]  ) ");
                sql.Append("SELECT MatrixKey," + mfieldname[0] + ", RANK() OVER(ORDER BY MatrixKey) AS [rank] FROM (" + matrixtablesql + ") as [ranktable]");
                AddProgressSQL("Populate the new intermediate fact table.", new string[] 
                    { "SQL query to populating the intemediate fact table." }
                    , sql.ToString(), MatrixTableName);
                if (!bGenerateScriptOnly)
                {
                    scmd.CommandText = sql.ToString();
                    scmd.ExecuteNonQuery();
                }
                //Update the matrix key in the intermediate dimension table.
                sql = new StringBuilder();
                sql.Append("UPDATE " + msTableName[(int)Tables.IntermediateDimension] + 
                    " SET [" + MatrixIntKeyName + "]=[" + MatrixTableName + "].[" + MatrixIntKeyName + "] ");
                sql.Append("FROM " + msTableName[(int)Tables.IntermediateDimension] + " JOIN (");
                sql.Append(matrixkeysql + ") AS Matrix ON ");
                for (int idx = 0; idx < nfieldname.Count; idx++)
                {
                    sql.Append("Matrix." + nfieldname[idx] + "=" + msTableName[(int)Tables.IntermediateDimension] +
                        "." + nfieldname[idx] + " ");
                    if (idx < nfieldname.Count - 1) sql.Append(" AND ");
                }
                sql.Append("join [" + MatrixTableName + "] on [" + MatrixTableName + "].[" +
                    MatrixRealKeyFieldName + "]=Matrix.MatrixKey");
                AddProgressSQL("SQL to update the matrix key in the intermediate dimension table.", new string[] { "Populate the intemediate dimension table matrix key." }
                    , sql.ToString(), msTableName[(int)Tables.IntermediateDimension]);
                if (mbIntDimIsReal)
                {
                    if (!bGenerateScriptOnly)
                    {
                        scmd.CommandText = sql.ToString();
                        scmd.ExecuteNonQuery();
                    }
                }
                else
                {
                    AddProgress("Cannot populate the intermediate dimension table.", "It must be a real table.", msTableName[(int)Tables.IntermediateDimension], ProgressIcons.Warning);
                }
                //
                if (!bGenerateScriptOnly)
                {
                    CleanUpSQLSession(scmd);
                    //
                    AddProgress("Commiting transaction on data source.", string.Empty, sconn.ConnectionString);
                    strans.Commit();
                    //
                    scmd.Dispose();
                    AddProgress("Successfully populated the relational matrix tables.", "Please be sure to process the appropriate SSAS objects.", string.Empty);
                }
                //
                return true;
            }
            catch (Exception ex)
            {
                if (strans != null)
                {
                    AddProgress("Rolling back transaction on data source.", string.Empty, string.Empty);
                    strans.Rollback();
                }
                AddProgress("Error populating the matrix tables.", ex.Message, "cmd_Populate_Click", ProgressIcons.Error);
                return false;
            }
            finally
            {
                CloseDataSourceConnection(sconn);
            }
        }

        internal bool ChangesApplied
        {
            get { return mbChangesApplied; }
        }
        private bool mbChangesApplied = false;
        //An extended property on ManyToManyMeasureGroupRelationship shows that this compression method
        //has been applied. This means the changes to the schema (in GenerateScripts) can't be run again.
        //again. However, 
        private const string ChangesAppliedExtendedProperty = "M2NMatrixLookupTable";

        /// <summary>
        /// The type of the key of the "many" dimension table.
        /// Note that this isn't the same as the MatrixIntKey, which is the surrogate key for the new matrix lookup table.
        /// </summary>
        internal string ManyKeyType
        {
            get { return manyKeyType; }
        }
        private string manyKeyType="int";


        internal bool GenerateScriptsOKtoRun(bool bGenerateScriptOnly)
        {
            return assessmentDone && !ShowStopper && bProperlyInited &&
                (!mbChangesApplied || (mbChangesApplied && bGenerateScriptOnly));
        }

        private void InitOperation()
        {
            miTaskCount = 0;
        }
        /// <summary>
        /// Generate scripts and/or apply changes to the relational schema and the 5OLAP cube schema.
        /// 
        /// The changes will be made only if the bbGenerateScriptsOnly flag=false and the AcceptableMaxItemsOnManySide=true.
        /// Otherwide, the scripts will be created and returned to the client via the AddProgress event.
        /// </summary>
        /// <param name="serv"></param>
        /// <param name="selectedcube"></param>
        /// <param name="bGenerateScriptOnly"></param>
        /// <param name="bRemoveOldIntermediateMG"></param>
        /// <returns></returns>
        internal bool GenerateScripts(Microsoft.AnalysisServices.Server serv,Microsoft.AnalysisServices.Cube selectedcube,bool bGenerateScriptOnly,bool bRemoveOldMG)
        {
            InitOperation();
            if (!GenerateScriptsOKtoRun(bGenerateScriptOnly)) return false;
            OleDbConnection sconn=null;
            OleDbTransaction strans=null;
            OleDbCommand scmd=null;
            bool bInSSASTransaction = false;
            try
            {
                if (!bGenerateScriptOnly)
                {
                    sconn = OpenDataSource();
                    if (sconn == null) return false;
                    strans = sconn.BeginTransaction();
                    scmd = new OleDbCommand();
                    scmd.CommandTimeout = miCommandTimeout;
                    scmd.Connection = sconn;
                    scmd.Transaction = strans;

                    //Now, actually commit the changes.
                    AddProgress("Begin transaction on SSAS server.", string.Empty, serv.Name);
                    serv.BeginTransaction();
                    /*
                     * TODO: Something strange happened with the SSAS transaction.
                    */
                    bInSSASTransaction = true;

                }
                //
                AddProgress("Beginning tasks for applying schema changes.", this.ToString(), "cmdGenerateScripts_Click");
                StringBuilder sql;
                //Get the data type for the many to many key. We need this to create the new matrix lookup table.
                //Warning: If bGenerateScriptOnly=true, GetSqlKeyType will try to open a connection. If it can't the field type will remain "int".
                manyKeyType = GetSqlKeyType(mplaintablename, mplainfieldname[0], sconn, strans,manyKeyType);
                //
                //Create the matrix table.
                //Notice that there is code in the SQL to check if the table already exists.
                sql = new StringBuilder();
                sql.Append("IF EXISTS (SELECT * FROM dbo.sysobjects WHERE Name='" + MatrixTableName + "' AND TYPE='u') DROP TABLE [" + MatrixTableName + "];");
                sql.Append("CREATE TABLE [" + MatrixTableName + "] ([" + MatrixIntKeyName + "] int,[" + MatrixRealKeyFieldName + "] varchar(" + MatrixHashKeyMaxLength.ToString() + ")," + mfieldname[0] + " " + manyKeyType + ");");
                AddProgressSQL("Create matrix table.", new string[] 
                    {
                        "Create matrix table.",
                        "M2M Relationship: "+this.ToString()
                    }, sql.ToString(), MatrixTableName);

                if (!bGenerateScriptOnly && AcceptableMaxItemsOnManySide)
                {
                    scmd.CommandText = sql.ToString();
                    scmd.ExecuteNonQuery();
                }
                //Populate the matrix table.
                //Add matrix key field to the dimension table.
                sql = new StringBuilder();
                sql.Append("IF NOT EXISTS(SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '" + interDimTablePlainName + "' AND COLUMN_NAME = '" + MatrixIntKeyName + "') ");
                sql.Append("ALTER TABLE " + msTableName[(int)Tables.IntermediateDimension] + " ADD [" + MatrixIntKeyName + "] int");
                AddProgressSQL("Add matrix key to the intermediate dimension table.", new string[] 
                    {
                        "Add matrix key to the intermediate dimension table.",
                        "M2M Relationship: "+this.ToString()
                    }, sql.ToString(), msTableName[(int)Tables.IntermediateDimension]);
                if (!bGenerateScriptOnly && AcceptableMaxItemsOnManySide)
                {
                    if (mbIntDimIsReal)
                    {
                        scmd.CommandText = sql.ToString();
                        scmd.ExecuteNonQuery();
                    }
                    else
                    {
                        AddProgress("The intermediate dimension table isn't a real table.", "You must add the field, [" + MatrixIntKeyName + "] int, to the database object (view)," + msTableName[(int)Tables.IntermediateDimension]+", before populating the relational database objects.", msTableName[(int)Tables.IntermediateDimension], ProgressIcons.Task);
                    }
                }
                //Create a script to undo the database changes.
                sql = new StringBuilder();
                sql.AppendLine("ALTER TABLE " + msTableName[(int)Tables.IntermediateDimension] + " DROP COLUMN [" + MatrixIntKeyName + "]");
                sql.AppendLine("GO");
                sql.AppendLine("DROP TABLE [" + MatrixTableName + "]");
                sql.AppendLine("GO");
                AddProgressSQL("Script to undo relational database changes.", new string[] 
                    {
                        SpecialSQLTag,
                        "1. Remove matrix key from the intermediate dimension table: "+msTableName[(int)Tables.IntermediateDimension]+".["+matrixIntKeyName+"]",
                        "2. Remove the matrix lookup table: ["+matrixTableName+"]"
                    }, sql.ToString(), msTableName[(int)Tables.IntermediateDimension]);
                //Populate the matrix key field in the dimension table.
                //Add int dim matrix key to dsv.
                if (!bGenerateScriptOnly)
                {
                    //
                    if (dsv.Schema.Tables[interdimTableID].Columns.Contains(MatrixIntKeyName))
                    {
                        AddProgress("Matrix integer key is already added to the DSV table.", "Be sure this column is correct.", MatrixIntKeyName, ProgressIcons.Informational);
                    }
                    else
                    {
                        AddProgress("Adding matrix integer key to the DSV table.", string.Empty, MatrixIntKeyName);
                        DataColumn dc = new DataColumn(MatrixIntKeyName, typeof(int));
                        dc.ExtendedProperties.Add("DbColumnName", MatrixIntKeyName);
                        dc.ExtendedProperties.Add("FriendlyName", MatrixIntKeyName);
                        dsv.Schema.Tables[interdimTableID].Columns.Add(dc);
                    }
                    //Add the matrix table to the DSV.
                    if (dsv.Schema.Tables.Contains(MatrixTableName))
                    {
                        AddProgress("Matrix table is already in the DSV.", "Be sure this table is correct.", MatrixTableName, ProgressIcons.Informational);
                    }
                    else
                    {
                        AddProgress("Adding matrix table to the DSV.", string.Empty, MatrixTableName);
                        //TODO: This new matrix table goes to dbo for now. But, I should later allow for a selection.
                        OleDbDataAdapter adapter = new OleDbDataAdapter("SELECT * FROM [" + DefaultRelationalSchema + "].[" + MatrixTableName + "] WHERE 1=0", sconn);
                        adapter.SelectCommand.Transaction = strans;
                        DataTable[] dataTables = adapter.FillSchema(dsv.Schema, SchemaType.Mapped, MatrixTableName);
                        DataTable dataTable = dataTables[0];
                        //
                        dataTable.ExtendedProperties.Add("TableType", "Table");
                        dataTable.ExtendedProperties.Add("DbSchemaName", DefaultRelationalSchema);
                        dataTable.ExtendedProperties.Add("DbTableName", MatrixTableName);
                        dataTable.ExtendedProperties.Add("FriendlyName", MatrixTableName);
                        foreach (DataColumn dc in dataTable.Columns)
                        {
                            dc.ExtendedProperties.Add("DbColumnName", dc.ColumnName);
                            dc.ExtendedProperties.Add("FriendlyName", dc.ColumnName);
                        }
                        //
                        AddRelation(MatrixTableName, MatrixIntKeyName, interdimTableID, MatrixIntKeyName);
                        AddRelation(MatrixTableName, mfieldname[0].Replace("[", string.Empty).Replace("]", string.Empty), dimTableID, dimKeyName);
                    }
                    //Add the new matrix field as an attribute of the intermediate dimension.
                    if (!IntermediateDimension.Dimension.Attributes.Contains(MatrixIntKeyName))
                    {
                        AddProgress("Add the new matrix field an as attribute of the intermediate dimension.", string.Empty, MatrixIntKeyName);
                        DimensionAttribute attr = IntermediateDimension.Dimension.Attributes.Add(MatrixIntKeyName);
                        attr.KeyColumns.Add(CreateDataItem(dsv, interdimTableID, MatrixIntKeyName));
                    }
                    //attr.NameColumn = CreateDataItem(dsv, interdimTableID, MatrixIntKeyName);


                    //Add the new matrix table as a measure group to the cube.
                    MeasureGroup newmg;
                    if (!selectedcube.MeasureGroups.Contains(MatrixTableName))
                    {
                        AddProgress("Adding new matrix table as a measure group in the cube.", string.Empty, MatrixTableName);
                        newmg = selectedcube.MeasureGroups.Add(MatrixTableName);
                        //Need to give this measure a name that must be unique across all measures.
                        Microsoft.AnalysisServices.Measure newm = newmg.Measures.Add("Count_" + MatrixTableName);
                        newm.AggregateFunction = AggregationFunction.Count;
                        newm.Source = new DataItem(MatrixTableName, MatrixIntKeyName, OleDbType.Integer);
                        //Create partition for the measure group.
                        Partition newpart = newmg.Partitions.Add(MatrixTableName);
                        //TODO: Need to allow selection of schema. For now, this table will be added to dbo.
                        newpart.Source = new TableBinding(dsv.DataSource.ID, DefaultRelationalSchema, MatrixTableName);
                        newpart.ProcessingMode = ProcessingMode.Regular;
                        newpart.StorageMode = StorageMode.Molap;
                        //Link the new matrix measure group to the dimension using Regular relationships.
                        RegularMeasureGroupDimension rmgd = newmg.Dimensions.Add(M2M.CubeDimensionID);
                        MeasureGroupAttribute mga = rmgd.Attributes.Add(M2M.Dimension.KeyAttribute.ID);
                        mga.Type = MeasureGroupAttributeType.Granularity;
                        DataItem dataitem = mga.KeyColumns.Add(MatrixTableName, mplainfieldname[0], OleDbType.Integer);
                        //Link the new matrix measure group to the intermediate dimension using Regular relationships.
                        rmgd = newmg.Dimensions.Add(IntermediateDimension.CubeDimensionID);
                        mga = rmgd.Attributes.Add(MatrixIntKeyName);
                        mga.Type = MeasureGroupAttributeType.Granularity;
                        mga.KeyColumns.Add(MatrixTableName, MatrixIntKeyName, OleDbType.Integer);
                    }
                    else
                    {
                        newmg = selectedcube.MeasureGroups[MatrixTableName];
                    }
                    //Save a reference to the old intermediate MG for removal later.
                    //The reason I'm getting it now is because Many2ManyMG is derived from MeasureGroupDimension
                    //and the reference will go invalid after the BaseMG.Dimensions.Remove(dimname) line.
                    //An error will show up later when attempting to remove the old M2M MG.
                    //I'm not exactly sure why that happens.
                    MeasureGroup oldIntMG = Many2ManyMG;
                    //Make the new matrix table measure group the intermediate fact table of the relationship.
                    string dimname = M2M.CubeDimensionID;
                    //First remove the old m2n relationship from the base measure group.
                    BaseMG.Dimensions.Remove(dimname);
                    //Add a new m2m for the dimension and the new intermediate fact table.
                    ManyToManyMeasureGroupDimension newmgd=new ManyToManyMeasureGroupDimension(dimname, newmg.ID);
                    //This annotation tells this application that the schema changes were already applied.
                    clsM2M.AddAnnotation((ModelComponent)newmgd,ChangesAppliedExtendedProperty,"Made through CompressManyToMany");
                    BaseMG.Dimensions.Add(newmgd);
                    if (bRemoveOldMG && selectedcube.MeasureGroups.Contains(oldIntMG))
                    {
                        AddProgress("Removing old many to many MG.", string.Empty, oldIntMG.Name);
                        selectedcube.MeasureGroups.Remove(oldIntMG);
                    }
                    else
                    {
                        AddProgress("Skipping removal old many to many MG.", string.Empty, oldIntMG.Name);
                    }
                }
                /*
                 * At this point, all changes are made and we begin a transaction to save the changes.
                */
                if (!bGenerateScriptOnly)
                {
                    /*
                    Before committing the transactions, I want to be sure the SQL to populate the 
                    tables are saved to the progress log.
                    This way, if the user were to leave without populating the tables, the SQL
                    is there. I could have made it such that the schema changes and population are done
                    at the same time, but it may take a long time.
                    
                    Anyway, the SQL to populate will be lost since it's based on the old SSAS schema. So,
                    we should save them anyway. To ensure we have them, I'm running PopulateMatrix with
                    the "GenerateScriptOnly" flag set to true.
                    */
                    AddProgress("Retrieving the SQL to populate the tables just in case the user exits before doing it.", string.Empty, this.ToString());
                    PopulateMatrix(true);

                    AddProgress("Updating changes to the data source view.", string.Empty, dsv.Name);
                    dsv.Update();
                    AddProgress("Updating changes to the intermediate dimension.", string.Empty, IntermediateDimension.Dimension.Name);
                    IntermediateDimension.Dimension.Update(UpdateOptions.ExpandFull);
                    AddProgress("Updating changes to the selected cube.", string.Empty, selectedcube.Name);
                    selectedcube.Update(UpdateOptions.ExpandFull);
                    //Now that the updates to both the data source and SSAS database are made, let's commit.
                    AddProgress("Commiting transaction on data source.", string.Empty, sconn.ConnectionString);
                    strans.Commit();
                    strans = null;
                    AddProgress("Committing transaction on SSAS server.", string.Empty, serv.Name);
                    serv.CommitTransaction();
                    bInSSASTransaction = false;
                    scmd.Dispose();
                    AddProgress("Successfully applied schema changes.", this.ToString(), this.ToString());
                }
                /*
                 * End of transaction phase. All updates are safely made at this time.
                */

                //Now that the schema changes are applied, we set a flag so this cannot be done again.
                if (!bGenerateScriptOnly)
                {
                    mbChangesApplied = true;
                }
                return true;
           
           }
            catch (Exception ex)
            {
                if (strans != null)
                {
                    AddProgress("Rolling back transaction on data source.", string.Empty, dsv.Name);
                    strans.Rollback();
                }
                if (bInSSASTransaction)
                {
                    AddProgress("Rolling back transaction on SSAS server.", string.Empty, serv.Name);
                    serv.RollbackTransaction();
                }
                AddProgress("Error during application of changes.", ex.Message, "cmdGenerate_Scripts_Click", ProgressIcons.Error);
                return false;
            }
            finally
            {
                CloseDataSourceConnection(sconn);
            }
           
            
        }
        /// <summary>
        /// This isn't a good way to get the schema information.
        /// </summary>
        /// <param name="tablename"></param>
        /// <param name="fieldname"></param>
        /// <param name="conn"></param>
        /// <param name="trans"></param>
        /// <returns></returns>
        private string GetSqlKeyType(string tablename, string fieldname, OleDbConnection conn, OleDbTransaction trans,string defaultvalue)
        {
            if (conn == null)
            {
                conn = OpenDataSource();
                //If a connection can't be open, return. 
                if (conn == null) return defaultvalue;
            }
            string keytype=defaultvalue;
            OleDbCommand scmd = OpenCommand("SELECT DATA_TYPE,CHARACTER_MAXIMUM_LENGTH FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '" + tablename + "' AND COLUMN_NAME = '" + fieldname + "'", conn);
            if (trans != null) scmd.Transaction = trans;
            OleDbDataReader dr = scmd.ExecuteReader();
            while (dr.Read())
            {
                keytype = dr["DATA_TYPE"].ToString();
                if (!(dr["CHARACTER_MAXIMUM_LENGTH"] is System.DBNull))
                {
                    keytype += "(" + dr["CHARACTER_MAXIMUM_LENGTH"].ToString() + ")";
                }
                break;      //Should have only one row.
            }
            dr.Close();
            return keytype;
        }
        private void AddRelation(string fkTableName, string fkColumnName, string pkTableName, string pkColumnName)
        {
            List<string> fkColumnNameList=new List<string>();
            fkColumnNameList.Add(fkColumnName);
            List<string> pkColumnNameList=new List<string>();
            pkColumnNameList.Add(pkColumnName);
            AddRelation(fkTableName, fkColumnNameList, pkTableName, pkColumnNameList);
        }
        private void AddRelation(string fkTableName, List<string> fkColumnName, string pkTableName, List<string> pkColumnName)
        {
            StringBuilder fkColStr = new StringBuilder();
            DataColumn[] fkColumn=new DataColumn[fkColumnName.Count];
            for (int idx=0;idx<fkColumnName.Count;idx++)
            {
                fkColumn[idx]=dsv.Schema.Tables[fkTableName].Columns[fkColumnName[idx]];
                fkColStr.Append(fkColumnName[idx]);
            }
            // when run with the parameters causing errors indicated above the following line 
            // results in an "object not initialized" error.  When breaking code execution before the error
            // it succeeds on the line above and dsv.Schema has a value and after the following line is
            // run dsv.Schema becomes null
            DataColumn[] pkColumn = new DataColumn[pkColumnName.Count];
            for (int idx = 0; idx < pkColumnName.Count; idx++)
            {
                pkColumn[idx]=dsv.Schema.Tables[pkTableName].Columns[pkColumnName[idx]];
            }
            dsv.Schema.Relations.Add("FK_" + fkTableName + "_" + fkColStr, pkColumn, fkColumn, true);
        }
        static internal bool AddAnnotation(ModelComponent amoobj,string name,string value)
        {
            if (amoobj.Annotations.Contains(name)) return false;
            Annotation item = new Annotation(name,value);
            amoobj.Annotations.Add(item);
            return true;
        }
        /// <summary>
        /// Convert a column from the DSV into a DataItem object.
        /// </summary>
        /// <param name="dsv"></param>
        /// <param name="tableName"></param>
        /// <param name="columnName"></param>
        /// <returns></returns>
        static private DataItem CreateDataItem(DataSourceView dsv, string tableName, string columnName)
        {
            DataTable dataTable = ((DataSourceView)dsv).Schema.Tables[tableName];
            DataColumn dataColumn = dataTable.Columns[columnName];
            return new DataItem(tableName, columnName, OleDbTypeConverter.GetRestrictedOleDbType(dataColumn.DataType));
        }
        private void AddProgress(string message, string details, string objname)
        {
            AddProgress(message, details, objname, ProgressIcons.None);
        }
        private void AddProgress(string message, string details, string objname, ProgressIcons state)
        {
            clsProgress prog = new clsProgress(message, details,state,objname);
            if (state == ProgressIcons.Warning || state == ProgressIcons.Error || state==ProgressIcons.Task || state==ProgressIcons.Informational)
            {
                Errors.Add(prog);
                miTaskCount++;
            }
            if (Progress!=null)
            {
                Progress(this, new ProgressEventArgs(prog));
            }
        }
        private void AddProgressSQL(string message, string[] tsqlcomments, string sql, string objname)
        {
            StringBuilder sb=new StringBuilder();
            foreach (string str in tsqlcomments)
            {
                sb.AppendLine(TSQLComment+str);
            }
            sb.AppendLine(sql);
            AddProgress(message, sb.ToString(), objname, ProgressIcons.SQL);
        }
        public static string FormatStringArray(List<string> items, string delimiter)
        {
            return FormatStringArray(string.Empty, items, delimiter);
        }
        public static string FormatStringArray(string preamble,List<string> items, string delimiter)
        {
            StringBuilder sb = new StringBuilder();
            for (int idx = 0; idx < items.Count; idx++)
            {
                sb.Append(preamble+items[idx]);
                if (idx < items.Count - 1) sb.Append(delimiter);
            }
            return sb.ToString();
        }
    }
    internal class ProgressEventArgs : EventArgs
    {
        private clsProgress progress;

        public ProgressEventArgs(clsProgress progress)
        {
            this.progress = progress;
        }
        public clsProgress Progress
        {
            get { return progress; }
        }
    }
    internal struct RankCount
    {
        internal RankCount(int rank, int count)
        {
            Rank = rank;
            Count = count;
        }
        internal int Rank;
        internal int Count;

        public string Description(string baseMGTableName,string manyDimTableName)
        {
            return "There are " + Count.ToString() + " occurences of rows in the 'base measure group', " + 
                baseMGTableName + ", related to " + Rank.ToString() + " rows in the 'many' dimension, " + 
                manyDimTableName;
        }
    }
    internal enum Tables
    {
        BaseMeasureGroup=0,
        IntermediateDimension=1,
        IntermediateMeasureGroup=2,
        ManyDimension=3
    }
}
